import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;


/** Hauptservlet der Artikelveraltung
  * Hierin wird die Artikelliste erzeugt, die Artikel gelscht und gespeichert
  *
  * @author Bernhard Fiser 0109815*/
public class ArtikelVerwaltungServlet extends HttpServlet
{
	private static ArtikelVerwaltung
			av;

	/** Servletinitialisierung beim erstmaligen Laden des Servlets
          * vom Tomcat */
	public void init()
	{
		try
		{
			/* Falls mglich AritkelVerwaltungsobjekt, welches die Artikel
			 * beinhaltet, laden */
		        ObjectInputStream objectInput = new ObjectInputStream(new FileInputStream("/home/itpt/a0109815/public_html/servlets/store/artikel.dat"));
			av = (ArtikelVerwaltung) objectInput.readObject();
			objectInput.close();
		}

		catch (Exception e)
		{
			/* Falls nicht mglich, dann eine neue, leere Artikelverwaltung
			 * erstellen */
			av = new ArtikelVerwaltung();
		}
	}

	/** Interne Methode zum speichern der Artikelveraltung
	  * @param PrintWriter out : Interface zum Webclient output
	  * @return true : falls Speichern erfolgreich
	  * @return false : falls Fehler aufgetreten ist */
	private boolean save(PrintWriter out)
	{
		try
		{
			/* Datei ffnen */
		        ObjectOutputStream objectOutput = new ObjectOutputStream(new FileOutputStream("/home/itpt/a0109815/public_html/servlets/store/artikel.dat"));
			/* Object speichern */
		    	objectOutput.writeObject(av);
			/* Datei schreiben und schlieen */
		        objectOutput.flush();
        		objectOutput.close();
		}
		/* Im Fehlerfall Exception ausgeben und false zurckgeben */
		catch (Exception e) { out.println("<font color=\"red\">" + e + "</font><br>"); return(false); }
		return(true);
	}

	/** Methode fr CGI-POST. Diese wird auf die doGet Methode
	  * weitergeleitet */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
	{
		this.doGet(request, response);
	}

	/** Methode fr CGI-GET. Hier findet die Verarbeitung der Daten statt */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
	{
		int	id,
			preis,
			menge,
			c;
		String	desc,
			url,
			bg;

		/* Contenttype fr output setzen */
		response.setContentType("text/html");
		/* Outputstream festhalten */
		PrintWriter out = response.getWriter();
		/* HTML-Doc Einleitung */
		out.println(
			"<html><head><meta name=\"author\" content=\"Bernhard Fiser\">" +
			"<meta name=\"date\" content=\"2002-05-14T08:28:00+02:00\">" +
			"<title>Artikelverwalung V1.0 - Bernhard Fiser 0109815</title></head><body>\n");

		/* Falls der CGI-Parameter 'cgi_save' bergeben wurde,
		 * Artikel in Artikelverwaltung aufnehmen und speichern */
		if (request.getParameter("cgi_save") != null)
		{
			try
			{
				/* CGI-Parameter aus FORM bernehmen und convertiertn */
				id = Integer.parseInt(request.getParameter("cgi_id"));
				preis = (int) (Double.parseDouble(request.getParameter("cgi_preis")) * 100);
				menge = Integer.parseInt(request.getParameter("cgi_menge"));
			}
			catch (NumberFormatException e)
			{
				/* Falls Konvertierung fehlgeschlagen, Werte auf 0 setzen */
				id = 0;
				preis = 0;
				menge = 0;
			}
			/* restliche CGI-Parameter bernehmen. */
			desc = request.getParameter("cgi_desc");
			url = request.getParameter("cgi_url");

			/* int Werte auf Plausibilitt prfen */
			if ((id > 0) && (preis > 0) && (menge > 0))
			{
				/* Falls Werte in Ordnung, neuen Artikel anlegen... */
				Artikel a = new Artikel(id, desc, preis, menge, url);
				/* ... und in Artikelverwaltun aufnehmen... */
				av.changeArtikel(a);
				/* ... und speichern */
				if (!this.save(out))
					out.println("<font color=\"red\"><b>Fataler Fehler beim Speichern!</b></font><br>");
				else
					/* Falls Speicherung erfolgreich, dann OK-Meldung ausgeben */
					out.println("<font color=\"green\">Artikel Nummer " + id + " wurde erfolgreich gespeichert!</font><br>");
			}
			else
				/* Falls Plausibilittsprfung fehlgeschlagen, Fehler ausgeben */
				out.println("<font color=\"red\">Artikel konnte nicht gespeichert werden! Eingabedaten sind falsch.</font><br>");
		}

		/* Falls CGI-Parameter 'cgi_del' bergeben wurde, Artikelobjekt
		 * aus der Artikelverwaltung lschen */
		if (request.getParameter("cgi_del") != null)
		{
			/* bergebene Artikelnummer konvertieren und auf
			 * Plausibilitt prfen */
			try { id = Integer.parseInt(request.getParameter("cgi_id")); }
			catch (NumberFormatException e) { id = 0; }
			if (id > 0)
			{
				/* Falls Plausibilitt OK, Artikel lschen */
				if (av.deleteArtikel(av.getArtikel(id)))
				{
					/* ArtikelVerwaltung speichern */
					if (!this.save(out))
						out.println("<font color=\"red\"><b>Fataler Fehler beim Speichern!</b></font><br>");
					else
						/* OK-Meldung ausgeben, falls Speicherung erfolgreich */
						out.println("<font color=\"green\">Artikel Nummer " + id + " wurde erfolgreich gel&ouml;scht.</font><br>");
				}
				else
					/* Falls Plausibilittsprfung fehlgeschlagen, Fehler ausgeben */
					out.println("<font color=\"red\"Artikel Nummer " + id + " konnte nicht gel&ouml;scht werden. Artikel existiert nicht.</font></br>");
			}
		}

		/* Button fr Neuanlage generiern*/
		out.println("<form method=\"post\" action=\"ArtikelDetailServlet\"><input type=\"submit\" value=\"Neuen Artikel anlegen\"></form><hr>");
		/* Tabelle der vorhandenen Artikel generieren */
		out.println("<table border=\"1\"><tr><th>ID</th><th>Beschreibung</th><th>Preis</th><th>Menge</th><th>URL</th></tr>");
		/* Iterator der Hashmap erstellen */
		Iterator it = av.iterator();
		c = 0;
		/* ber jedes Objekt iterieren */
		while (it.hasNext())
		{
			/* Hintergrundfarbe der Tabelle festlegen */
			if ((c++ % 2) == 0)
				bg = "#aaeeaa";
			else
				bg = "#eeeeaa";
			/* Artikel auslesen aus der Hashmap */
			Artikel a = (Artikel) it.next();
			/* Zeile mit den Artikelattribute ausgeben */
			out.println(
				"<tr bgcolor=\"" + bg + "\">" +
				"<td>" + a.getID() + "</td>" +
				"<td>" + a.getDescription() + "</td>" +
				"<td>" + (double) a.getPreis() / 100 + "</td>" +
				"<td>" + a.getMenge() + "</td>" +
				"<td><a href=\"" + a.getURL() + "\">" + a.getURL() + "</a></td>" +
				"<td><a href=\"ArtikelDetailServlet?cgi_id=" + a.getID() + "\">&Auml;ndern</a></td>" +
				"<td><a href=\"ArtikelVerwaltungServlet?cgi_id=" + a.getID() + "&cgi_del=1\">L&ouml;schen</a></td>" +
				"</tr>\n");
		}
		/* Tabelle und HTML-Doc beenden */
		out.println("</table></body></html>\n");
	}

}
